<?php
// *** 请注意文件字符集应为 UTF-8 ***
// echo __FILE__, "#", __LINE__, "<br/>\n"; // 调试语句
//
// ----------------------------------------------------------------------------
setlocale(LC_CTYPE, "UTF8", "en_US.UTF-8");

///---INTERFACE-METHODS---BEGIN---///
function zfs_snapshot_info(){
    // 引入全局变量，此处不要修改
    global $request;
    global $respond;

    if ( ! isset($request->params) || !(gettype($request->params) == "array" || gettype($request->params) == "object") )
    {
        $respond->err    = ERR_PARAMS;
        $respond->errmsg = "缺少参数:params{zpool,zfilesys}";
    }else
    if ( !validString($request->params->zpool) )
    {
        $respond->err    = ERR_PARAMS;
        $respond->errmsg = "缺少参数或参数内容错误:zpool";
    }else
    if ( !validString($request->params->zfilesys) )
    {
        $respond->err    = ERR_PARAMS;
        $respond->errmsg = "缺少参数或参数内容错误:zfilesys";
    }else{
        $zfs = \UNAS\Volume\Volume::CreateVolumeObject("zfs");
        $zpool = $request->params->zpool;
        $zfilesys = $request->params->zfilesys;

        $snapshots = $zfs->GetSnapshot($zpool, $zfilesys);

        $respond->results = [
            "autosnap" => autosnap_info($zpool, $zfilesys),
            "folders" => share_folders(),
            "data" => $snapshots,
            "replicationTasks" => file_exists(ZFS_FILE_REPLICATION) ? json_decode(file_get_contents(ZFS_FILE_REPLICATION), true) : [],
            "running" => snap_running_tasks($zpool, $zfilesys),
        ];
        $respond->err = ERR_SUCCEED;
    }
    return true;
}

function autosnap_info(string $zpool, string $zfilesys){
    $autoSnapshotJson = array();
    if(file_exists(ZFS_FILE_AUTOSNAP)){
        $autoSnapshotJson = json_decode(file_get_contents(ZFS_FILE_AUTOSNAP), true);
    }
    $zfs_path = "$zpool/$zfilesys";
    if (isset($autoSnapshotJson[$zfs_path])){
        $autoSnapshotJson[$zfs_path]["status"] = true;
        return $autoSnapshotJson[$zfs_path];
    }
    return ["status" => false];
}

function snap_running_tasks($zpool, $zfilesys) {
    $send = [];
    foreach (glob(ZFS_DIR_RUN_SEND_CONF."/*") as $send_conf) {
        $task_name = basename($send_conf);
        $task_conf = @json_decode(file_get_contents($send_conf), true);
        if (isset($task_conf["name"]) && $task_conf["zfs"] == "$zpool/$zfilesys") {
            $status_cmd = sprintf(ZFS_CMD_SNAPSHOT_SEND_STATUS, $task_name);
            exec($status_cmd, $content, $ret);
            if ($ret === 0) {
                $task_conf["progress"] = decode_progress_file(ZFS_DIR_RUN_SEND_PROGRESS."/$task_name");
                $send[] = $task_conf;
            }
        }
    }

    $replicate = [];
    if(file_exists(ZFS_FILE_REPLICATION)){
        $replicationTasksJson = json_decode(file_get_contents(ZFS_FILE_REPLICATION), true);
        foreach ($replicationTasksJson as $task_name => $conf) {
            if ($conf["dataset"] !== "$zpool/$zfilesys") {
                continue;
            }
            $status_cmd = sprintf(ZFS_CMD_REPLICATE_STATUS, $task_name);
            exec($status_cmd, $content, $ret);
            if ($ret === 0) {
                $replicate[] = ["name" => $task_name, "progress" => decode_progress_file(ZFS_DIR_RUN_REPLICATE_PROGRESS."/$task_name")];
            }
        }
    }

    return ["send" => $send, "replicate" => $replicate];
}

function decode_progress_file($progress_file) {
    $info = [];
    $lines = @explode(PHP_EOL, file_get_contents($progress_file));
    if (isset($lines[0])) {
        $info["size"] = intval($lines[0]);
        if (sizeof($lines) > 2) {
            $progress_now = explode(" ", $lines[sizeof($lines) - 2]);
            $info["time"] = intval($progress_now[0]);
            $info["progress"] = intval($progress_now[1]);
            $info["percent"] = $info["progress"] / $info["size"];
            $info["speed_avg"] = $info["progress"] / $info["time"];
            $info["eta"] = ($info["size"] - $info["progress"]) * $info["time"] / $info["progress"];
            if (sizeof($lines) > 3) {
                // Calculate last second speed
                $info["speed"] = $info["progress"] - intval(explode(" ", $lines[sizeof($lines) - 3])[1]);
            } else {
                // Not enough data for speed
            }
        }
    }
    return $info;
}

?>
