<?php
// *** 请注意文件字符集应为 UTF-8 ***
// echo __FILE__, "#", __LINE__, "<br/>\n"; // 调试语句
//
// ----------------------------------------------------------------------------
setlocale(LC_CTYPE, "UTF8", "en_US.UTF-8");

///---INTERFACE-METHODS---BEGIN---///
function zfs_zpool_create(){
    // 引入全局变量，此处不要修改
    global $request;
    global $respond;

    if ( ! isset($request->params) || !(gettype($request->params) == "array" || gettype($request->params) == "object") )
    {
        $respond->err    = ERR_PARAMS;
        $respond->errmsg = "缺少参数:params{zpool,pvs,raidLevel,spares,logs,cache,ashift}";
    }else
    if ( !validString($request->params->zpool) )
    {
        $respond->err    = ERR_PARAMS;
        $respond->errmsg = "缺少参数或参数内容错误:zpool";
    }else
    if ( !is_array($request->params->pvs) )
    {
        $respond->err    = ERR_PARAMS;
        $respond->errmsg = "缺少参数或参数内容错误:pvs";
    }else
    if ( !validString($request->params->raidLevel) )
    {
        $respond->err    = ERR_PARAMS;
        $respond->errmsg = "缺少参数或参数内容错误:raidLevel";
    }else
    if ( !is_array($request->params->spares) )
    {
        $respond->err    = ERR_PARAMS;
        $respond->errmsg = "缺少参数或参数内容错误:spares";
    }else
    if ( !is_array($request->params->logs) )
    {
        $respond->err    = ERR_PARAMS;
        $respond->errmsg = "缺少参数或参数内容错误:logs";
    }else
    if ( !is_array($request->params->cache) )
    {
        $respond->err    = ERR_PARAMS;
        $respond->errmsg = "缺少参数或参数内容错误:cache";
    }else
    if ( !is_int($request->params->ashift) )
    {
        $respond->err    = ERR_PARAMS;
        $respond->errmsg = "缺少参数或参数内容错误:ashift";
    }else{
        $zfs = \UNAS\Volume\Volume::CreateVolumeObject("zfs");
        $zpool = $request->params->zpool;
        $pvs = $request->params->pvs;
        $raidLevel = $request->params->raidLevel; //0-strip, 1-mirror, 5-raidz1, 6-raidz2, 10, raidz3
        $spares = $request->params->spares;
        $logs = $request->params->logs;
        $cache = $request->params->cache;
        $dedup = asArray($request->params->dedup);
        $special = asArray($request->params->special);
        $ashift = $request->params->ashift;

        if (!($zpool && $pvs)){
            $respond->err = ERR_NO_INPUT;
        }else
        if($zfs->VolumeGroupExists($zpool)){
            $respond->err = ERR_USED;
        }else{
            $pvs_in = [];
            foreach ($pvs as $pv){
                if (\UNAS\Disk\InitializeDisk($pv)){
                    $pvs_in[] = dev_id($pv);
                }
            }

            if (empty($spares)){
                $attr = array(
                    "raid" => $raidLevel,
                );
            }else{
                $pvs_spare = [];
                foreach ($spares as $pv){
                    if (\UNAS\Disk\InitializeDisk($pv)){
                        $pvs_spare[] = dev_id($pv);
                    }
                }
                $attr = array(
                    "raid" => $raidLevel,
                    "spare" => $pvs_spare
                );
            }

            if (!empty($logs)){
                $pvs_log = [];
                foreach ($logs as $pv){
                    if (\UNAS\Disk\InitializeDisk($pv)){
                        $pvs_log[] = dev_id($pv);
                    }
                }
                $attr["log"]=$pvs_log;
            }

            if (!empty($cache)){
                $pvs_cache = [];
                foreach ($cache as $pv){
                    if (\UNAS\Disk\InitializeDisk($pv)){
                        $pvs_cache[] = dev_id($pv);
                    }
                }
                $attr["cache"]=$pvs_cache;
            }

            if (!empty($dedup)){
                $pvs_dedup = [];
                foreach ($dedup as $pv){
                    if (\UNAS\Disk\InitializeDisk($pv)){
                        $pvs_dedup[] = dev_id($pv);
                    }
                }
                $attr["dedup"]=$pvs_dedup;
            }

            if (!empty($special)){
                $pvs_special = [];
                foreach ($special as $pv){
                    if (\UNAS\Disk\InitializeDisk($pv)){
                        $pvs_special[] = dev_id($pv);
                    }
                }
                $attr["special"]=$pvs_special;
            }

            $attr['ashift'] = $ashift;

            $ret0 = $zfs->CreateVolumeGroup($zpool,$pvs_in, $attr);
            if($ret0){
                $eventDes = "##$@zfssuc@$##Successfully using device##$@zfssuc@$##".' "'.implode(',', $pvs_in).'" '."##$@zfssuc@$##to create Volume Group##$@zfssuc@$##".' "'.$zpool.'"'."##$@zfssuc@$##.##$@zfssuc@$##";
                appLog("Information", $eventDes);
                $respond->err = ERR_SUCCEED;
            }
            else{
                $eventDes = "##$@zfsfail@$##Using physical device##$@zfsfail@$##".' "'.implode(',', $pvs_in).'" '."##$@zfssuc@$##to create Volume Group##$@zfssuc@$##".' "'.$zpool.'" '."##$@zfsfail@$##failed##$@zfsfail@$##";
                appLog("Information", $eventDes);
                $respond->err = ERR_FAILED;
            }
        }
    }
    return true;
}
$dev_id_list = [];
function dev_id(string $name){
    global $dev_id_list;
    if (empty($dev_id_list)){
        foreach (glob("/dev/disk/by-id/*") as $id_raw){
            if (is_link($id_raw)){
                $dev_id_list[basename(realpath($id_raw))] = basename($id_raw);
            }
        }
    }
    $devname = basename($name);
    if (isset($dev_id_list[$devname])){
        return $dev_id_list[$devname];
    }else{
        return $name;
    }
}
?>
