<?php
// *** 请注意文件字符集应为 UTF-8 ***
// echo __FILE__, "#", __LINE__, "<br/>\n"; // 调试语句
//
// ----------------------------------------------------------------------------
setlocale(LC_CTYPE, "UTF8", "en_US.UTF-8");

///---INTERFACE-METHODS---BEGIN---///
function zfs_zpool_info(){
    // 引入全局变量，此处不要修改
    global $request;
    global $respond;

    if ( ! isset($request->params) || !(gettype($request->params) == "array" || gettype($request->params) == "object") )
    {
        $respond->err    = ERR_PARAMS;
        $respond->errmsg = "缺少参数:params{zpool}";
    }else
    if ( !validString($request->params->zpool) )
    {
        $respond->err    = ERR_PARAMS;
        $respond->errmsg = "缺少参数或参数内容错误:zpool";
    }else{
        //$zfs = \UNAS\Volume\Volume::CreateVolumeObject("zfs");
        $zpool = $request->params->zpool;

        $size = (int)exec(sprintf(ZFS_CMD_ZPOOL_SIZE, $zpool), $content, $ret) / 1024;
        if ( $ret == 0 )
        {
            $level = zpoolMemberType($zpool, $zdevice);

            $respond->results = [
                "name" => $zpool,
                "type" => "zPool",
                "size" => $size,
                "free" => (int)exec(sprintf(ZFS_CMD_ZFS_AVAIL_SIZE, $zpool)) / 1024,
                "health" => exec(sprintf(ZFS_CMD_ZPOOL_HEALTH, $zpool)),
                "status" => 1,
                "level"  => $level,
            ];
            $respond->results = array_merge($respond->results, scrub_status($zpool));

            $respond->results["pool_status"] = zpool_status($zpool);

            exec('zpool list -Ho dedup ' . $zpool, $output, $err);
            if ( ! $err && count($output)>0 )
            {
                $respond->results["dedup"] = trim($output[0]);
            }
        }else
        {
            $respond->results = [
                "name" => $zpool,
                "type" => "zPool",
                "size" => 0,
                "free" => 0,
                "health" => "Not imported",
                "status" => 0,
            ];
        }
        $respond->results["folders"] = share_folders();
        $respond->err = ERR_SUCCEED;
    }
    return true;
}

/**
 * Get params from a config line.
 *
 *  Cautions: The params may contain quotation marks.
 *  Cautions: All letters after '#' will be regard as comment.
 * @param string $config_line   The config line.
 * @return array                [$0, $1, $2...]
 */
function GetParams(string $config_line){
    // Replace special chars
    $patterns = ['/\(/','/\)/','/\*/'];
    $replacement = ['\(','\)','\*'];
    $cmd = 'printf "%s\n" '.preg_replace($patterns, $replacement, $config_line);
    exec($cmd, $content, $ret);

    return $content;
}

/**
 * Get scrub info of this $zpool
 *
 * @param string $zpool
 * @return array
 */
function scrub_status(string $zpool){
    $status = [ "status" => false ];
    foreach ( explode("\n", file_get_contents(ZFS_FILE_CRON_SCRUB)) as $scrub_job ){
        $line_conf = GetParams($scrub_job);
        if (end($line_conf) == $zpool){
            $status["status"] = true;
            $status["hour"] = (int)$line_conf[1];
            if ($line_conf[0] == 0 && $line_conf[3] == "*"){
                // 0 0 * * 0            every week
                // 0 0 8-14,22-28 * 6   every 2 weeks
                // 0 0 1 * *            every month
                if ($line_conf[2] == "*" && $line_conf[4] == "0"){
                    $status["interval"] = 0;
                }else
                if ($line_conf[2] == "8-14,22-28" && $line_conf[4] == "6"){
                    $status["interval"] = 1;
                }else
                if ($line_conf[2] == "1" && $line_conf[4] == "*"){
                    $status["interval"] = 2;
                }
            }
            break;
        }
    }
    return [ "scrub" => $status ];
}

function leadCount($line){
    $leadSpace = 0; //without \t
    if(strlen($line)>0){ // && $line[0] === '\t' , seem not be able to test \t
        for ($i = 1; $i < strlen($line); $i++){
            if($line[$i] ==' '){
                $leadSpace++;
                continue;
            }
            break;
        }
    }
    return $leadSpace;
}

function zpool_status(string $zpool){
    exec("zpool status " . $zpool, $st, $ret);
    $poolStatus=array();

    $isMirror = 0;
    for($i=0; $i < count($st); $i++){
        $line = trim($st[$i]);
        if (empty($line)) continue;
        if (strpos($line, 'pool:') !== false){
            $poolStatus['pool']= trim(str_replace("pool:", "",trim($line)));
            continue;
        }
        if (strpos($line, 'state:') !== false){
            $poolStatus['state']= trim(str_replace("state:", "",trim($line)));
            continue;
        }
        if (strpos($line, 'scan:') !== false){
            $poolStatus['scan']= trim(str_replace("scan:", "",trim($line)));
            continue;
        }
        if (strpos($line, 'errors:') !== false){
            $poolStatus['errors']= trim(str_replace("errors:", "",trim($line)));
            continue;
        }
        if (strpos($line, 'config:') !== false){
            $poolStatus['config']= array();
            $i =$i + 1 ; //skip the empty line and the "NAME STATE READ WRITE CKSUM" line, 2 lines after config:
            continue;
        }
        if (leadCount($st[$i]) == 2 ){
            if(strpos($line, 'mirror') !== false){ //process mirror device
                $isMirror = 1;
            }else{
                $isMirror = 0;
            }
        }
        if (strpos($line, 'CKSUM') !== false){ //process 2/stripped or 3/mirror or raid lines
            $currentPoolComponent= 'pool';  // reached the NAME STATUE READ WRITE CKSUM line， this indicates the pool.
            //next line is pool.
            $i++; //next line is pool
            $poolStatus['config'][] = array(
                "isDevice" => 0,
                "isMirrorDevice" => $isMirror,
                "leadCount" => leadCount($st[$i]),
                "poolComponent" => $currentPoolComponent,
                "line" => $st[$i]
            );
            continue;
        }
        if (strpos($line, 'dedup') !== false){ //dedup
            $currentPoolComponent= 'dedup';  // reached the NAME STATUE READ WRITE CKSUM line， this indicates the pool.
            $poolStatus['config'][] = array(
                                            "isDevice" => 0,
                                            "isMirrorDevice" => $isMirror,
                                            "leadCount" => leadCount($st[$i]),
                                            "poolComponent" => $currentPoolComponent,
                                            "line" => $st[$i]
                                        );
            continue;
        }
        if (strpos($line, 'special') !== false){ //special
            $currentPoolComponent= 'special';  // reached the NAME STATUE READ WRITE CKSUM line， this indicates the pool.
            $poolStatus['config'][] = array(
                                            "isDevice" => 0,
                                            "isMirrorDevice" => $isMirror,
                                            "leadCount" => leadCount($st[$i]),
                                            "poolComponent" => $currentPoolComponent,
                                            "line" => $st[$i]
                                        );
            continue;
        }
        if (strpos($line, 'logs') !== false){ //logs
            $currentPoolComponent= 'logs';  // reached the NAME STATUE READ WRITE CKSUM line， this indicates the pool.
            $poolStatus['config'][] = array(
                                            "isDevice" => 0,
                                            "isMirrorDevice" => $isMirror,
                                            "leadCount" => leadCount($st[$i]),
                                            "poolComponent" => $currentPoolComponent,
                                            "line" => $st[$i]
                                        );
            continue;
        }
        if (strpos($line, 'cache') !== false){ //cache
            $currentPoolComponent= 'cache';  // reached the NAME STATUE READ WRITE CKSUM line， this indicates the pool.
            $poolStatus['config'][] = array(
                "isDevice" => 0,
                "isMirrorDevice" => $isMirror,
                "leadCount" => leadCount($st[$i]),
                "poolComponent" => $currentPoolComponent,
                "line" => $st[$i]
            );
            continue;
        }
        if (strpos($line, 'draid') !== false){ //draid
            $currentPoolComponent= 'draid';
            $poolStatus['config'][] = array(
                "isDevice" => 0,
                "isMirrorDevice" => $isMirror,
                "leadCount" => leadCount($st[$i]),
                "poolComponent" => $currentPoolComponent,
                "line" => $st[$i]
            );
            continue;
        }
        if (strpos($line, 'spares') !== false){ //spare
            $currentPoolComponent= 'spares';  // reached the NAME STATUE READ WRITE CKSUM line， this indicates the pool.
            $poolStatus['config'][] = array(
                "isDevice" => 0,
                "isMirrorDevice" => $isMirror,
                "leadCount" => leadCount($st[$i]),
                "poolComponent" => $currentPoolComponent,
                "line" => $st[$i]
            );
            continue;
        }
        if(isDeviceLine($st[$i])){
            $by_path = "";
            //$d = explode(" ", trim($line));
            //if(count($d)>0){
            //    //first item is device id (cannot handle partition.)
            //    if(isDiskName($d[0])){
            //        $by_path = getByPathFromDevName($d[0]);
            //    }else{
            //        $by_path=$d[0];
            //    }
            //}
            $name = explode(" ", trim($line))[0];
            $poolStatus['config'][] = array(
                "isDevice" => preg_match("/(mirror-[\d])|(raidz[\d]-[\d])/",$name) ? false : true,
                "name" => $name,
                "isMirrorDevice" => $isMirror,
                "leadCount" => leadCount($st[$i]),
                "poolComponent" => $currentPoolComponent,
                "line" => $st[$i],
            );
            continue;
        }
        // when we are here, somthing we don't know showed up...
        if(strlen($line) <=0 ){
            $currentPoolComponent= 'unknown';  // empty line, change currentPoolComponent
        }else{
            if (isset($poolStatus['extra_info'])){
                $poolStatus['extra_info'][] = $line;
            }else{
                $poolStatus['extra_info'] = [$line];
            }
        }
    }
    return $poolStatus;
}

function isDeviceLine(string $line){
    if ($line[0] != "\t") return false;
    $parts = preg_split('/\s+/', trim($line));
    $dev = $parts[0];
    if ($dev == "NAME" || count($parts) < 2) return false;

    if (($parts[1] == "ONLINE" || $parts[1] == "OFFLINE" || $parts[1] == "REMOVED")
    && preg_match("/[\d]+/", $parts[2])
    && preg_match("/[\d]+/", $parts[3])
    && preg_match("/[\d]+/", $parts[4])
    )
    {
        return true;
    }

    // In fact will not be here
    if (preg_match("/(mirror-[\d])|(raidz[\d]-[\d])/",$dev)) return false;
    $possible_links = [
        "/dev/$dev",
        "/dev/disk/by-id/$dev",
        "/dev/disk/by-partuuid/$dev",
        "/dev/disk/by-path/$dev",
        "/dev/disk/by-uuid/$dev",
    ];

    foreach ($possible_links as $try_link){
        if (is_link($try_link)){
            return true;
        }
    }
    return false;
}

function zpoolMemberType(string $name, &$device)
{
    $ret = '';
    exec("zpool status -j --json-flat-vdevs $name", $output, $err);
    if ( !$err )
    {
        $json = json_decode($output[0]);
        $pools = (array)$json->pools;
        $poolArray = (array)$pools[$name];
        $vdevsArray = (array)$poolArray['vdevs'];
        if ( ! empty($vdevsArray) )
        {
            $ret = 'stripe';
            foreach( $vdevsArray as $devid=>$obj )
            {
                $classify = substr($devid, 0, 7);
                if ( $classify == "draid1:"
                  || $classify == "draid2:"
                  || $classify == "draid3:" )
                {
                    $device = $obj->name;
                    $ret = substr($obj->name, 0, 6);
                }else
                if ( $obj->vdev_type == "mirror" )
                {
                    $device = $obj->name;
                    $ret = $obj->vdev_type;
                }else
                if ( $obj->vdev_type == "raidz" )
                {
                    $device = $obj->name;
                    $ret = substr($obj->name, 0, 6);
                }
            }
        }
    }
    return $ret;
}

?>
