<?php

// SQLite 数据库操作相关

class FS2SQLite
{
const SQL_TABLE_EXISTS = <<<MULTI_STRING
SELECT * FROM SQLITE_MASTER WHERE type = 'table' AND UPPER(name) = UPPER(?) LIMIT 1;
MULTI_STRING;

const SQL_READ_DBVERSION =
<<<MULTI_STRING
SELECT *
  FROM t_configure
 WHERE UPPER(cfg_key) = UPPER('Database.Version')
 LIMIT 1;
MULTI_STRING;

static public function dbTableExists($pdo, $tableName)
{
    $ret = false;
    if ( self::dbExecSQL($rs, $pdo, self::SQL_TABLE_EXISTS, [$tableName]) >= 0 )
    {
        if ( $row = self::dbNext($rs) )
        {
            $ret = true;
        }
    }
    return $ret;
}

static public function dbOpen($database)
{
    // 只读数据库会出现无法正常操作的现象
    exec("sudo chmod 777 " . pathinfo($database, PATHINFO_DIRNAME));
    if ( is_file($database) )
    {
        exec("sudo chmod 777 " . $database);
    }
    return new PDO('sqlite:' . $database, SQLITE3_OPEN_READWRITE);
}

static public function dbClose(&$pdo)
{
    unset($pdo);
}

static public function dbExecTransaction($pdo, $sqls)
{
    $ret = false;
    try {
        $pdo->beginTransaction();
        $ret = true;
        $pdo->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);
        foreach( $sqls as $sql )
        {
            $query = $pdo->prepare($sql);
            if ( $query )
            {
                $query->execute();
                $query->setFetchMode(PDO::FETCH_ASSOC);
            }else
            {
                $ret = false;
                break;
            }
        }
        if ( $ret )
        {
            $pdo->commit();
        }else
        {
            $pdo->rollBack();
        }
    }catch(PDOException $e) {
        $ret = false;
        $pdo->rollBack();
    }
    return $ret;
}

static public function dbExecSQL(&$rs, $pdo, $sql, $params=[], &$lastId=NULL) // 参数按?顺序赋值
{
    $ret = -1;
    try {
        $pdo->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);
        $query = $pdo->prepare($sql);
        if ( $query )
        {
            $query->execute($params);
            $query->setFetchMode(PDO::FETCH_ASSOC);
            $rs = $query;
            $ret = $query->rowcount();

            $lastId = $pdo->lastInsertId();
        }else
        {
            $rs = "sql error.";
        }
    } catch(PDOException $e) {
        $rs = $e->getMessage();
    }
    return $ret;
}

static public function dbNext($rs)
{
    $ret = [];
    $row = $rs->fetch();
    if ( $row )
    {
        foreach( $row as $key=>$value )
        {
            $ret[strtolower($key)] = $value;
        }
    }
    return $ret;
}

static public function dbResult($rs)
{
    $ret = 0;
    if ( $row = $rs->fetch() )
    {
        foreach($row as $k=>$v)
        {
            $ret = $v;
            break;
        }
    }
    return $ret;
}

static public function readDatabaseVersion($pdo)
{
    $ret = -1;
    if ( self::dbExecSQL($rs, $pdo, self::SQL_READ_DBVERSION) >= 0 )
    {
        if ( $row = self::dbNext($rs) )
        {
            $ret = (int)$row["cfg_value"];
        }
    }
    return $ret;
}


} // End of class

?>
