<?php
// 注意文件字符集统一为 UTF-8
//echo __FILE__, "#", __LINE__, "<br/>\n"; // 调试语句

require_once("fs2error.php");
require_once("common.php");
require_once("unasutils.php");
require_once("settings.php");

$allSettings = glob(dirname(__FILE__) . '/settings/*.php');

$g_session = NULL;

foreach ($allSettings as $filename)
{
    if ( is_file($filename) )
    {
        require_once $filename;
    }
}

function startSession($token="")
{
    $ret = false;

    $_COOKIE = [ 'PHPSESSID' => '' ]; // 强制禁止浏览器的隐式cookie中的sessionId

    global $g_session;

    require_once('usssession.php'); // 引入源文件

    $g_session = new USSSession($token);
    if ( ($g_session->elapsed() + $g_session->tokenDuration()) >= NSSession::elapsedSeconds() )
    {
        $g_session->renewal();
        $ret = true;
    }else
    {// 需要删除么？可以先不删？
        session_destroy();
    }
    // false: session 已失效
    return $ret;
}

function sessionRenewal()
{
    $ret = false;

    global $g_session;

    if ( isLoggedIn() )
    {
        $g_session->renewal();
        $ret = true;
    }

    return $ret;
}

function listToParams($params)
{
    global $tagging;
    global $request;
    global $respond;

    $request = (object)[
        "params"=>[],
    ];
    foreach( $params as $k=>$v )
    {
        switch($k)
        {
        case "cmd":
            $request->cmd = $v;
            break;
        case "token":
            $request->token = $v;
            break;
        default:
            $request->params[$k] = $v;
            break;
        }
    }
    $request->params = (object)$request->params;
}

function callRequst($indexDir)
{
    global $tagging;
    global $request;
    global $respond;

    if ( validString($request->cmd) )
    {
        $phpCmdFile = $indexDir . "/cmds/" . $request->cmd . ".php";
        if ( is_file($phpCmdFile) )
        {
            try {
                /// 此处不能使用 require_once , 它无法被 try ... catch 捕获
                @include_once($indexDir . "/cmds/" . $request->cmd . ".php");
                if ( @call_user_func($request->cmd) )
                {
                    $requestD = glob(dirname(__FILE__) . '/request.d/*.php');
                    foreach ($requestD as $phpfile)
                    {
                        @include_once($phpfile);
                    }
                }else
                {
                    errorSystem();
                }
            } catch(Exception $err) {
                $respond->err    = ERR_SYSTEM;
                $respond->errmsg = $err->getMessage();
            }
        }else
        {
            $method = "fsi_" . $request->cmd;
            if ( in_array($method, get_defined_functions()["internal"]) )
            {
                try {
                    // 仅为方便 C++侧处理，传入的 json 转为 string 再传入
                    $respond = @call_user_func($method, json_encode($request));
                    if ( is_array($respond) || is_object($respond) )
                    {
                        $respond = (object) $respond;
                    }else
                    {
                        $respond = (object)[];
                        errorSystem();
                    }
                    $requestD = glob(dirname(__FILE__) . '/request.d/*.php');
                    foreach ($requestD as $phpfile)
                    {
                        @include_once($phpfile);
                    }
                } catch(Exception $err) {
                    $respond->err    = ERR_SYSTEM;
                    $respond->errmsg = $err->getMessage();
                }
            }else
            if ( defined("LPC_PASSAGE") )
            {
                $passage = LPC_PASSAGE;
                if ( validString($request->passage) )
                {
                    $passage = $request->passage;
                }
                if ( FS2LpcRequest($passage, $request, $retObject) )
                {
                    $respond = $retObject;
                }else
                {
                    //$respond->err = ERR_SYSTEM;
                    errorSystem();
                }
            }else
            {
                errorSystem();

                // 本地直接与代理程序由进程通讯方式的接口调用。是作为未来版本而引入的先进工作方式。现已调试成功，暂不开通使用
                // $request->OVERSTEP = 1; // 僭越模式，不检查调用者身份; 仅限于LS方式时可操作; 不推荐的调用方式
                // //FS2RpcRequest(LPC_GLOBALID, $request, $respond, asInt($request->timeout, LPC_TIMEOUT));
                // FS2LpcRequest(LPC_GLOBALID, $request, $respond, asInt($request->timeout, LPC_TIMEOUT));
            }
        }
        session_write_close();
    }else
    {
        errorParams();
    }
}

function startup($indexDir, $timezone="PRC", $showError="Off")
{
    ini_set("display_errors", $showError); // 倘若无法正常运行，打开屏蔽可显示出错位置
    date_default_timezone_set($timezone); // UTC 时区, PRC 中国时区
    setlocale(LC_CTYPE, "UTF8", "en_US.UTF-8"); // 设置当前默认的字符集

    // 启用SQLite Session机制
    //FS2PhpSession::init(SESSION_DATABASE);
    //FS2SessionFile::init();

    //测试用，有安全隐患
    header("Access-Control-Allow-Origin: *");
    header("Access-Control-Allow-Headers: Origin, X-Requested-With, Content-Type, Accept");

    // 引入全局变量，此处不要修改
    global $tagging;
    global $request;
    global $respond;
    global $g_session;

//    $g_mySession = new NSSession();

    $tagging = (object)[
        "notreply"=>false,
    ];
    $respond = (object) array("err"=>ERR_FAILED);
//$starttime=date();
    if ( $_SERVER["REQUEST_METHOD"] == "POST" )
    {
        $postdata = file_get_contents("php://input");

        if ( ! empty($postdata) )
        {
            $request = json_decode($postdata);
            if ( isset($request) && (gettype($request) == "array" || gettype($request) == "object") )
            {
                if ( validString($request->cmd) )
                {
                    startSession(trim(asString($request->token)));
                    if ( $request->cmd == "pingpong"  // 检查是否超时
                      || $request->cmd == "noop" )    // 空载
                    {
                        if ( isLoggedIn() )
                        {
                            $respond->err = ERR_SUCCEED;
                        }else
                        {
                            $respond->err = ERR_TIMEOUT;
                        }
                    }else
                    {
                        //sessionRenewal();
                        if ( isset($request->params) )
                        {
                            $request->params = (object)$request->params;
                        }else
                        {
                            $request->params = (object)[];
                        }
                        callRequst($indexDir);
                    }
                }else
                {
                    errorParams();
                }
            }else
            if ( isString($_POST["cmd"]) )
            {
                startSession(asString($_POST["token"]));
                sessionRenewal();
                listToParams($_POST);
                callRequst($indexDir);
            }else
            {
                errorParams();
            }
        }else
        {
            $request = (object)[];
            $cmd = $_POST['cmd'];
            if ( validString($cmd) )
            {
                sessionRenewal();
                if ( $cmd === "file_upload" )
                {
                    if ( ! isLoggedIn() )
                    {//检查是否已经登录
                        $respond->err    = ERR_NOT_LOGGED_IN;
                        $respond->errmsg = "尚未登录";
                    }else
                    if ( ! validString($_POST["module"]) )
                    {
                        $respond->err    = ERR_PARAMS;
                        $respond->errmsg = "缺少参数或参数内容错误:module";
                    }else
                    if ( ! validString($_POST["filename"]) )
                    {
                        $respond->err    = ERR_PARAMS;
                        $respond->errmsg = "缺少参数或参数内容错误:filename";
                    }else
                    if ( ! isset($_FILES) || ! isset($_FILES["file"]) )
                    {
                        $respond->err    = ERR_PARAMS;
                        $respond->errmsg = "缺少参数或参数内容错误:file";
                    }else
                    {
                        $request->cmd    = trim($_POST["module"]);
                        $request->token  = trim($_POST["token"]);
                        $request->params = (object)[
                            "filename" => trim($_POST["filename"]),
                            "fileinfo" => (object)$_FILES["file"],
                        ];
                        startSession(asString($request->token));
                        callRequst($indexDir);
                    }
                }else
                if ( $cmd === "apps_upload" )
                {
                    $request->cmd = "apps_upload";
                    $request->token  = $_POST["token"];
                    $request->params = (object)[
                        "name" => $_POST["name"],
                    ];
                    if ( isset($_FILES) && isset($_FILES["file"]) )
                    {
                        $request->params->fileinfo = (object)$_FILES["file"];
                    }
                    startSession(asString($request->token));
                    callRequst($indexDir);
                }else
                {
                    $respond->err    = ERR_PARAMS;
                    $respond->errmsg = "The lack of instructions and parameters.[1]";
                }
            }else
            {
                $respond->err    = ERR_PARAMS;
                $respond->errmsg = "The lack of instructions and parameters.[2]";
            }
        }
    }else
    if ( $_SERVER["REQUEST_METHOD"] == "GET" )
    {
        if ( isString($_GET["cmd"]) )
        {
            startSession(asString($_GET["token"]));
            sessionRenewal();
            listToParams($_GET);
            callRequst($indexDir);
        }else
        {
            errorParams();
        }
    }else
    {
        $respond->protocol = "FriendSafe.i";
        $postdata = '';
        $respond->err    = ERR_SYSTEM;
        $respond->errmsg = "POST only.";
        //$respond->results = $_SERVER;
    }
//$endtime=date();
//logTrack($starttime, $endtime);
    if ( ! $tagging->notreply )
    {
        //echo json_encode($respond, JSON_UNESCAPED_UNICODE | JSON_PRETTY_PRINT);
        echo json_encode($respond, JSON_UNESCAPED_UNICODE);
    }
}


function logTrack($starttime, $endtime)
{
    // 引入全局变量，此处不要修改
    global $request;
    global $respond;

    list($msec, $sec) = explode(" ", microtime());
    $msec = (int)($msec*1000);
    while(strlen($msec)<3)
    {
        $msec = '0' . $msec;
    }
    $nowtime = date("Ymdhis") . $msec;

    $cmd = "none_cmd";
    if ( validString($request->cmd) )
    {
        $cmd = $request->cmd;
    }

    $content =
        "StartTime: " . $starttime . "\n" .
        "EndTime: " . $endtime . "\n" .
        "Time  : " . date("Y-m-d h:i:s") . "." . $msec ." \n" .
        "Client: " . $_SERVER['REMOTE_ADDR'] . " \n\n" .
        "==== request " . str_repeat("=", 80) . "\n" .
        json_encode($request, JSON_UNESCAPED_UNICODE | JSON_PRETTY_PRINT) . "\n\n\n" .
        "==== respond " . str_repeat("=", 80) . "\n" .
        json_encode($respond, JSON_UNESCAPED_UNICODE | JSON_PRETTY_PRINT) . "\n\n\n" .
        "==== SESSION " . str_repeat("=", 80) . "\n";

    if ( isset($_SESSION) && (is_object($_SESSION) || is_array($_SESSION)) )
    {
        $content .=
            json_encode($_SESSION, JSON_UNESCAPED_UNICODE | JSON_PRETTY_PRINT);
    }else
    {
        $content .= "{}";
    }
    $content .= "\nisLoggedIn(): " . (isLoggedIn() ? "true" : "false");

    $content .= "\n\n";

    $logsDir = dirname(__FILE__) . "/logs/";
    //@exec("/usr/bin/sudo mkdir -p " . $logsDir);
    $filePath = $logsDir . $nowtime . "." . $cmd . ".log";
    file_put_contents($filePath, $content);
//echo __FILE__, "#", __LINE__, "<br/>\n"; // 调试语句
//echo $filePath, "\n";
}

?>
