<?php

require_once('/unas/api/php/UNASlog.php');

function isActivated()
{
    $request = [
        "cmd" => "wylicense_info",
    ];

    return UnasLpcRequest('.ls.nascore.agent.', $request, $retObject)
        && $retObject->err == ERR_SUCCEED;
}

function applog($logLevel, $eventDes, $appid=APPID)
{
    $userName = isset($_SESSION['username']) ? $_SESSION['username'] : "(unknown)";
    $ipAddr   = GetUserIP();
    $log = new \UNAS\Log\UnasLog();
    $log->WriteLog($logLevel, $eventDes, $appid, $userName, $ipAddr);
}

function writeAppLog($app, $logLevel, $eventDes)
{
    $userName = isset($_SESSION['username']) ? $_SESSION['username'] : "(unknown)";
    $ipAddr   = GetUserIP();
    $log = new \UNAS\Log\UnasLog();
    $log->WriteLog($logLevel, $eventDes, $app, $userName, $ipAddr);
}

function fileLoad($filePath)
{
    $exists = fileChmod($filePath, "666", $fperm);
    $ret = file_get_contents($filePath);
    if ( $exists )
    {
        fileChmod($filePath, $fperm);
    }
	return $ret;
}

function fileSave($filePath, $data)
{
    $exists = fileChmod($filePath, "666", $fperm);
    $ret = file_put_contents($filePath, $data);
    if ( $exists )
    {
        fileChmod($filePath, $fperm);
    }
	return $ret;
}

function fileChmod($filePath, $newmod, &$oldmod=null)
{
    $ret = false;
    if ( file_exists($filePath) )
    {
        $oldmod = substr(decoct(fileperms($filePath)), -4, 4);
        exec("/usr/bin/sudo /bin/chmod $newmod $filePath 2>/dev/null");
        $ret = true;
    }
    return $ret;
}

/**
 * Join the paths. Act like os.path.join() in python.
 *
 * Cautions: This can't handle http paths or any path contains double-slash("//").
 *
 * e.g.
 *      join_paths("/mnt/abc", "b") => "/mnt/abc/b"
 *      join_paths("/mnt/abc", "/b") => "/mnt/abc/b"
 *      join_paths("/mnt/abc/", "/b") => "/mnt/abc/b"
 *      join_paths("mnt/abc", "b") => "mnt/abc/b"
 */
function join_paths() {
    $paths = array();

    foreach (func_get_args() as $arg) {
        if ($arg !== '') { $paths[] = $arg; }
    }

    return preg_replace('#/+#','/',join('/', $paths));
}

function FS2RpcRequest($serverName, $cmdObject, &$retObject=null, $timeout=30)
{
    $ret = false;
    $socket = socket_create(AF_UNIX, SOCK_STREAM, 0);
    if ( $socket )
    {
        if ( @socket_connect($socket, "/tmp/" . $serverName) )
        {
            socket_set_option($socket, SOL_SOCKET, SO_SNDTIMEO, ["sec"=>$timeout, "usec"=>0]);
            socket_set_option($socket, SOL_SOCKET, SO_RCVTIMEO, ["sec"=>$timeout, "usec"=>0]);
            $cmdData = json_encode($cmdObject);
            $in  = pack("C1", 0);
            $in .= "fs";
            $in .= pack("C1", 1);
            $in .= pack("C1", 1);
            $in .= pack("V1", strlen($cmdData));
            $in .= pack("V1", 0);
            $in .= pack("C3", 0xFF, 0xFF, 0xFF);
            $in .= $cmdData;
            if ( socket_write($socket, $in, strlen($in)) )
            {
                $receivedBytes = "";
                while($received=socket_read($socket, 8192, PHP_BINARY_READ))
                {
                    $receivedBytes .= $received;
                    if ( strlen($receivedBytes) >= 14 && substr($receivedBytes, 0, 3) === "\0fs" )
                    {
                        $cmdSizes = unpack("V*", substr($receivedBytes, 5, 4));
                        if ( strlen($receivedBytes) >= 16 + $cmdSizes[1] )
                        {
                            $json = substr($receivedBytes, 16, $cmdSizes[1]);
                            $retObject = json_decode($json);
                            if ( $retObject )
                            {
                                $ret = true;
                            }
                            break;
                        }
                    }
                }
            }
        }
        socket_close($socket);
    }
    return $ret;
}

function UnasLpcRequest($serverName, $cmdObject, &$retObject=null, $timeout=30)
{
    $ret = false;
    $socket = socket_create(AF_UNIX, SOCK_STREAM, 0);
    if ( $socket )
    {
        if ( strspn($serverName, "/") > 0 )
        {
            $listenId = $serverName;
        }else
        {
            $listenId = "/run/unas/localsockets/" . $serverName;
        }
        if ( @socket_connect($socket, $listenId) )
        {
            socket_set_option($socket, SOL_SOCKET, SO_SNDTIMEO, ["sec"=>$timeout, "usec"=>0]);
            socket_set_option($socket, SOL_SOCKET, SO_RCVTIMEO, ["sec"=>$timeout, "usec"=>0]);
            $cmdData = json_encode($cmdObject);
            $in  = pack("C1", 0);
            $in .= "fs";
            $in .= pack("C1", 1);
            $in .= pack("C1", 1);
            $in .= pack("V1", strlen($cmdData));
            $in .= pack("V1", 0);
            $in .= pack("C3", 0xFF, 0xFF, 0xFF);
            $in .= $cmdData;
            if ( socket_write($socket, $in, strlen($in)) )
            {
                $receivedBytes = "";
                while($received=socket_read($socket, 8192, PHP_BINARY_READ))
                {
                    $receivedBytes .= $received;
                    if ( strlen($receivedBytes) >= 14 && substr($receivedBytes, 0, 3) === "\0fs" )
                    {
                        $cmdSizes = unpack("V*", substr($receivedBytes, 5, 4));
                        if ( strlen($receivedBytes) >= 16 + $cmdSizes[1] )
                        {
                            $json = substr($receivedBytes, 16, $cmdSizes[1]);
                            $retObject = json_decode($json);
                            if ( $retObject )
                            {
                                $ret = true;
                            }
                            break;
                        }
                    }
                }
            }
        }
        socket_close($socket);
    }
    return $ret;
}

function etcConfigLoad($filePath)
{
    $ret = [];
    $lines = file($filePath);
    foreach( $lines as $line )
    {
        $str = trim($line);
        if ( strlen($str) >0 && substr($str, 0, 1) != "#" )
        {
            $pos = strpos($str, "=");
            if ( $pos >0 )
            {
                $key   = substr($str, 0, $pos);
                $value = substr($str, $pos, strlen($str)-$pos);
                $cmdline = 'printf "%s\n" ' . $value;
                exec($cmdline, $output, $err);
                if ( ! $err && count($output)>0 )
                {
                    $value = $output[0];
                }
                $ret[$key] = trim($value);
            }
        }
    }
    return $ret;
}

?>


