<?php

/// @brief USS 专用会话管理，可同时支持浏览器与非浏览器的客户端
/// @note 使用前需要确保 ussfpm 服务正常运行中: systemctl status ussfpm
/// @code
///     // 应当在源码入口处尽早调用
///     require_once('usssession.php'); // 引入源文件
///     new USSSession(/*<TOKEN>*/);    // 可以指定token值，或缺省让系统自动生成
/// @endcode
class USSSession extends SessionHandler
{
public function __construct(string $token="")
{
    ini_set('session.serialize_handler', 'php_serialize');

    session_set_save_handler($this, true);

    $token = trim($token);
    if ( strlen($token) <= 0 && isset($_COOKIE['PHPSESSID']) && strlen($_COOKIE['PHPSESSID']) >0 )
    {
        $token = $_COOKIE['PHPSESSID'];
    }

    $this->m_object = new NSSession();

    if ( strlen($token) >0 )
    {
        $this->m_object->setToken($token);
        session_id($token);
    }
    session_start([
        'cookie_lifetime' => 86400,
        'read_and_close'  => false,
    ]);

    if ( strlen($token) <= 0 )
    {
        $token = session_id();
        $this->m_object->setToken($token);
    }

    setcookie('PHPSESSID', $token, 0);
}

public function elapsedSeconds(): int
{
    return NSSession::elapsedSeconds();
}

public function token()
{
    return $this->m_object->token();
}

public function setToken($token)
{
    $this->m_object->setToken($token);
}

public function elapsed()
{
    return $this->m_object->elapsed();
}

public function renewal()
{
    $this->m_object->renewal();
}

public function tokenDuration()
{
    $this->m_object->tokenDuration();
}

public function setTokenDuration(int $seconds)
{
    $this->m_object->setTokenDuration($seconds);
}

// override parent class:

public function create_sid(): string
{
    return $this->m_object->generate();
}

public function read($key): string | false
{
    $res = (array)$this->m_object->get();
    $ret = serialize($res);
    return $ret;
}

public function write($key, $data): bool
{
    $json = unserialize($data);
    return $this->m_object->rewrite($json);
}

public function destroy($key): bool
{
    return $this->m_object->destory();
}

private $m_object = NULL;

} // End of class USSSession

?>
